package meerkat.voting.gui.panels.select_candidate_name;

import com.google.common.util.concurrent.FutureCallback;
import com.google.protobuf.ByteString;
import javafx.event.Event;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;
import meerkat.protobuf.BallotQuestionUIElementOuterClass;

import java.util.Objects;


/**
 * Created by Vladimir Eliezer Tokarev on 10/2/2016.
 * This object gets the current stage and appends to the container of the names
 * the different names that there are in the config object and saves the use answer
 */
class StringsAnswersUpdater implements javafx.event.EventHandler{
    private Stage currentStage;
    private int rowAmount;
    private FutureCallback<String> updateAnswer;

    StringsAnswersUpdater(Stage primaryStage) {
        this.currentStage = primaryStage;
        this.rowAmount = 0;
        // The lookup works only after the css have been randered
        this.currentStage.getScene().getRoot().applyCss();
    }

    /**
     * The update answers method will be called every time the user select different answer
     * @param updateAnswer the method that will get the new answer every time
     */
    public void SetUpdateAnswers(FutureCallback<String> updateAnswer){
        this.updateAnswer = updateAnswer;
    }

    /**
     * Gets the container of the answers
     * @return GridPane object
     */
    private GridPane GetAnswersContainer(){
        return (GridPane) this.currentStage.getScene().lookup("#AnswersGridPane");
    }

    /**
     * Remove all previous answers from the container
     */
    private void RemoveAllAnswers(){
        this.GetAnswersContainer().getChildren().removeAll();
    }

    /**
     * Creates answer element which is grid pane with the answer and check box
     * @param answer string answer to show to the voter
     * @return GridPane which contains string and check box
     */
    private GridPane GetAnswerElement(String answer){
        GridPane gridPane = new GridPane();

        Label label = new Label();
        label.setText(answer);
        label.setPrefSize(550, 30);
        GridPane.setConstraints(label, 1, 1);

        CheckBox checkBox = new CheckBox();
        checkBox.setId(answer+"-checkBox");
        checkBox.setOnAction(this);
        GridPane.setConstraints(checkBox, 10, 1);

        gridPane.getChildren().addAll(label, checkBox);
        return  gridPane;
    }

    /**
     * Adds the GridPane answer to the panel
     * @param answer is the answer string to represent to the user
     */
    private void AddAnswer(String answer){
        GridPane container = this.GetAnswersContainer();

        GridPane newAnswer = this.GetAnswerElement(answer);
        container.addRow(0);

        this.rowAmount++;
        container.add(newAnswer, 0, this.rowAmount);
    }

    /**
     * Gets all the answers from the ballot ui question and puts them into the answers container
     * @param question
     */
    void UpdateAnswers(BallotQuestionUIElementOuterClass.BallotQuestionUIElement question) {
        this.RemoveAllAnswers();
        for (ByteString bytesAnswer : question.getAnswers().getAnswers().getAnswersList()){
            this.AddAnswer(bytesAnswer.toStringUtf8());
        }
        this.currentStage.show();
    }

    /**
     * Unchecks all  the check boxes that are not target box
     * @param target
     */
    private void uncheckBoxes(CheckBox target){
        GridPane answersContainer = this.GetAnswersContainer();
        for (Node child : answersContainer.getChildren()){
            GridPane answer = (GridPane)child;
            CheckBox  checkBox = (CheckBox) answer.getChildren().get(1);
            if (!Objects.equals(checkBox.getId(), target.getId()) && checkBox.isSelected()){
                checkBox.fire();
            }
        }

    }

    @Override
    /**
     * Saves the user chose
     */
    public void handle(Event event) {
        if (event.getSource() instanceof CheckBox) {
            CheckBox target = ((CheckBox)event.getSource());

            if (target.isSelected()) {
                String answer = target.getId().split("-")[0];
                this.updateAnswer.onSuccess(answer);
                this.uncheckBoxes(target);
            }
        }
    }

}
